module Main where

import Data.List

{-

Scale the training/testing data.

Reads a  file with the one line header at the front,
         <num_entries> <inputs> <outputs>
and scales both the inputs and outputs to the range  -1 < r < +1.

Scaling the entire file insures that the scale factors
are the same for both the training and testing.

-}

-----------------
--   Parsing   --
-----------------

parseLine :: String -> [Double]
parseLine s = map (\w -> (read w)::Double) (words s)

toTuples :: [(String,String)] -> [([Double],[Double])]
toTuples xs = map (\(a,b) -> (parseLine a, parseLine b)) xs

intoPairs :: [String] -> [(String,String)]
intoPairs [] = []
intoPairs (x:y:xs) = (x,y) : intoPairs xs


--------------------
--     Scaling    --
--------------------

scaleList :: [Double] -> [Double]
scaleList (x:[]) = [0] -- only one in list
scaleList xs = 
  if ((theMax - theMin) < 1e-10) then error "Col Diversity Error" -- hack /0
   else
      -- old version
      -- map (\x -> (-1) + (x - theMin) * (2.0 / (theMax - theMin))) xs
      -- new version
      map (\x -> (2*x - theMax -theMin)/(theMax - theMin)) xs
    where theMin = minimum xs
          theMax = maximum xs

scaleTuples :: [([Double],[Double])] -> [([Double],[Double])]
scaleTuples ds =
    zip (transpose (map (\x -> scaleList x) ins))
        (transpose (map (\x -> scaleList x) outs))
      where ins  = transpose . fst $ unzip ds
            outs = transpose . snd $ unzip ds

scaleData :: FilePath -> IO (String, [([Double],[Double])])
scaleData path = do 
  fileData <- (readFile path)
  hdr <- return $ head (take 1 (lines fileData))
  return $  (hdr, (scaleTuples . toTuples . intoPairs . (drop 1) $  (lines fileData)))


writeScaledTuples :: FilePath -> String -> [([Double],[Double])] -> IO ()
writeScaledTuples fileName hdr tuples = 
  (writeFile fileName) . unlines $ [hdr] ++ (map flatten tuples)
    where flatten (i,o) = (foldl (\cum s -> cum ++ show s ++ " ") "" i) ++ 
                          "\n" ++
                          (foldl (\cum s -> cum ++ show s ++ " ") "" o)

--------------
--   Main   --
--------------

main :: IO ()
main = do

  putStrLn "Scale Training/Testing Data (0.1) .."

  -- Read and Scale File Data
  (hdr,testTuples) <- scaleData "zoo.data"
  writeScaledTuples "scaled_zoo.data" hdr testTuples

  putStrLn "Done .."

